@@ -7,71 +7,93 @@ make.cem <- function(CM, status=FALSE, out=FALSE) {
 #   CM: a Community Matrix
 #   i:  a starting parameter
 #   j:  an ending parameter
-	enumerate.paths <- function(CM,i,j) {
+enumerate.paths <- function(CM,i,j) {
 
+	if (!is.numeric(i)) {
+		if (is.na(match(i,rownames(CM)))) {
+			stop("\nvalue \"", i, "\" of i is not the name of a parameter in the community matrix!\n")
+			}
+		match(i, rownames(CM)) -> i
+		}
+		
+	if (!is.numeric(j)) {
+		if (is.na(match(j,rownames(CM)))) {
+			stop("\nvalue \"", j, "\" of j is not the name of a parameter in the community matrix!\n")
+			}
+		match(j, rownames(CM)) -> j
+		}
+
+	validate.i.j <- function(i,j,N) {
+		if ( !(floor(i)==i) | !(floor(j)==j) | i > N | j > N | i < 1 | j < 1) {
+			stop("\ni and j must be integer values between 1 and ", N, ", or the names of the \ncommunity matrix parameters!")
+			}
+		# end validate.i.j
+		}
+
+ 
 # is.path tests that a list of visited elements terminates in the jth
 # parameter. It returns TRUE or FALSE and takes:
 # LOVE: a list of visited elements
 # j: a parameter that terminates a path from i to j
 
-		is.path <- function(LOVE,j) {
-			if (LOVE[length(LOVE)] == j) {
-				return(TRUE)
-				} else {
-			 	return(FALSE)
-			 	}
-
-			# end is.path()
-			} 
+	is.path <- function(LOVE,j) {
+		if (LOVE[length(LOVE)] == j) {
+			return(TRUE)
+			} else {
+		 	return(FALSE)
+		 	}
+		 	
+		# end is.path()
+		} 
 
 
 # make.ENVY() returns a list of Elements Not Visted Yet (ENVY) from:
 # CM: a valid Community Matrix
 # Term: a valid Termination matrix
 # LOVE: a list of visited elements within CM 
-		make.ENVY <- function() {
+	make.ENVY <- function() {
 
-			ENVY <- NULL
-			for (x in 1:N) {
-				if (!(identical((CM[LOVE[length(LOVE)],x]),0))) {
-					if (identical(Term[LOVE[length(LOVE)],x],0)) {
-						ENVY <- c(ENVY,x)
-						}
+		ENVY <- NULL
+		for (x in 1:N) {
+			if (!(identical((CM[x,LOVE[length(LOVE)]]),0))) {
+				if (identical(Term[x,LOVE[length(LOVE)]],0)) {
+					ENVY <- c(ENVY,x)
 					}
 				}
-			for (x in LOVE) {
-				ENVY <- ENVY[ENVY != x]
-				}
-
-			return(ENVY)
-
-			# end make.ENVY()	
 			}
+		for (x in LOVE) {
+			ENVY <- ENVY[ENVY != x]
+			}
+
+		return(ENVY)
+				
+		# end make.ENVY()	
+		}
 		
 	# update.Term() clears rows in the termination matrix Term of any
 	# parameters not in the List Of Visited Elements (LOVE). It
 	# modifies global Term:
-		update.Term <- function() {
-			for (x in 1:N) {
-				if (!(x %in% LOVE)) {
-					Term[x,c(1:N)] <<- 0
-					}
+	update.Term <- function() {
+		for (x in 1:N) {
+			if (!(x %in% LOVE)) {
+				Term[c(1:N),x] <<- 0
 				}
-			# end update.Term()
 			}
+		# end update.Term()
+		}
 		
 	# add.LOVE.to.LOP adds the List Of Visited Elements (LOVE) to the
 	# List Of Paths (LOP) and returns LOP. It takes:
 	# LOVE: a list of visited elements
 	# LOP: a list of paths
-		add.LOVE.to.LOP <- function() {
-			if (is.null(LOP)) {
-				LOP <<- list(LOVE)
-				} else {
-			 			LOP <<- c(LOP,list(LOVE))
-			 			}
-			 # end add.LOVE.to.LOP()
-			 }
+	add.LOVE.to.LOP <- function() {
+		if (is.null(LOP)) {
+			LOP <<- list(LOVE)
+			} else {
+		 			LOP <<- c(LOP,list(LOVE))
+		 			}
+		 # end add.LOVE.to.LOP()
+		 }
 
 	# search.step is the main logic of the breadth-wide path search. It
 	# returns a list of paths (LOP), and takes:
@@ -80,73 +102,76 @@ make.cem <- function(CM, status=FALSE, out=FALSE) {
 	# LOVE: a list of visited elements within CM
 	# LOP: a list of paths within CM 
 	# j: a parameter within CM that terminates a path from i to j
-		search.step <- function() {
-			ENVY <- make.ENVY()
-			# when there are no unvisited elements & LOVE has more than 1 element, 
-			# terminate the last element of LOVE for the second to last element of 
-			# LOVE.
-			if (length(ENVY) == 0) {
-				Term[LOVE[length(LOVE)-1],LOVE[length(LOVE)]] <<- 1
+	search.step <- function() {
+		ENVY <- make.ENVY()
+		# when there are no unvisited elements & LOVE has more than 1 element, 
+		# terminate the last element of LOVE for the second to last element of 
+		# LOVE.
+		if (length(ENVY) == 0) {
+			Term[LOVE[length(LOVE)],LOVE[length(LOVE)-1]] <<- 1
+			LOVE <<- LOVE[-length(LOVE)]
+		   update.Term()
+			# exit SearchStep if the last element of LOVE is i & i is terminated
+			# or if LOVE is empty and return List Of Paths (LOP)
+			if (length(LOVE) == 0 ) {
+				incomplete <<- FALSE
+				}
+			if (length(LOVE) == 1) {
+				test <- FALSE
+				for (val in c(1:N)[-i]) {
+					if (Term[val,i]==TRUE) {
+						test <- TRUE
+						}
+					}
+				if (test) {
+					return()
+					}
+				Term[1:N,LOVE[length(LOVE)]] <<- 1
 				LOVE <<- LOVE[-length(LOVE)]
 			   update.Term()
-				# exit SearchStep if the last element of LOVE is i & i is terminated
-				# or if LOVE is empty and return List Of Paths (LOP)
-				if (length(LOVE) == 0 ) {
-					incomplete <<- FALSE
-					}
-				if (length(LOVE) == 1) {
-					test <- FALSE
-					for (val in c(1:N)[-i]) {
-						if (Term[i,val]==TRUE) {
-							test <- TRUE
-							}
+				incomplete <<- FALSE
+				}
+			} else {
+					# append the first element (breadthwise search) of ENVY to LOVE
+		 			LOVE <<- c(LOVE,ENVY[1])
+					# test whether new LOVE is a path from i to j and respond accordingly
+					if (is.path(LOVE,j) == TRUE) {
+						add.LOVE.to.LOP()
+						Term[LOVE[length(LOVE)],LOVE[length(LOVE)-1]] <<- 1
+						LOVE <<- LOVE[-length(LOVE)]
 						}
-					if (test) {
-						return()
-						}
-					Term[LOVE[length(LOVE)],1:N] <<- 1
-					LOVE <<- LOVE[-length(LOVE)]
-				   update.Term()
-					incomplete <<- FALSE
-					}
-				} else {
-						# append the first element (breadthwise search) of ENVY to LOVE
-			 			LOVE <<- c(LOVE,ENVY[1])						# test whether new LOVE is a path from i to j and respond accordingly
-						if (is.path(LOVE,j) == TRUE) {
-							add.LOVE.to.LOP()
-							Term[LOVE[length(LOVE)-1],LOVE[length(LOVE)]] <<- 1
-							LOVE <<- LOVE[-length(LOVE)]
-							}
 						}
 	
-			# end search.step()
-			}
+		# end search.step()
+		}
 	
-		# Set N = rowsize of CM
-		N <- nrow(CM)
+	# Set N = rowsize of CM
+	N <- nrow(CM)
 
-		 # take care of the simple case of i = j
-		 if (identical(i,j)) {
-		 		LOP <- list(c(i,j))
-		 		return(LOP)
-		   	}
+	validate.i.j(i,j,N)
 
-		# initialize search termination matrix
-		Term <- matrix(c(0),N,N)
+	 # take care of the simple case of i = j
+	 if (identical(i,j)) {
+	 		LOP <- list(c(i,j))
+	 		return(LOP)
+	   	}
 
-		 # initialize list of paths (LOP), list of visited elements (LOVE)
-		 LOP <- NULL
-		 LOVE <- c(i)
+	# initialize search termination matrix
+	Term <- matrix(c(0),N,N)
 
-		incomplete <- TRUE
-		while (incomplete) {
-			search.step()
-			}
+	 # initialize list of paths (LOP), list of visited elements (LOVE)
+	 LOP <- NULL
+	 LOVE <- c(i)
+	 
+	incomplete <- TRUE
+	while (incomplete) {
+		search.step()
+		}
  	
- 		return(LOP)
-
-	 	# end enumerate.paths()
- 		}
+ 	return(LOP)
+ 	
+ 	# end enumerate.paths()
+ 	}
 
 
 # path.compliment takes as its input a path vector (Path) from i to j and returns the 
@@ -162,6 +187,24 @@ make.cem <- function(CM, status=FALSE, out=FALSE) {
 		}
 
 
+# looplist.elements returns a list of all variables appearing in
+# loops in LISTSET. It takes:
+# LOOPLIST: a list of loops, for example PLOS.
+
+	looplist.elements <- function(LOOPLIST) {
+		if (length(LOOPLIST) == 0) {
+			return(NULL)
+			}
+		elements <- NULL
+		for (q in 1:length(LOOPLIST)) {
+			elements <- c(elements,LOOPLIST[[q]])
+			}
+		return(unique(elements))
+	
+		# end looplist.elements
+		}
+
+
 # sum.path.x.C returns a sign sum of paths times the feedback of their 
 # complimentary subsystems. It takes:
 # CM: a community matrix
@@ -318,11 +361,17 @@ enumerate.SOSL <- function(MOSL,N) {
 			# end search.over()
 			}
  
+		# next.loop searches each list k of each column of a row in MOSL. If the list is:
+		# 1. not terminated, 
+		# 2. not NA, 
+		# 3. less than or equal to the length of N - PLOS, and
+		# 4. candidate loop does not contain elements in PLOS, then
+		# next.loop returns that list.
 		next.loop <- function(row) {
 
 			for (j in 1:length(MOSL[[row]])) {
 				for (k in 1:length(MOSL[[row]][[j]])) {
-					if ((Term[[row]][[j]][[k]] == 0) & (!is.na(MOSL[[row]][[j]][[k]][[1]])) & (length(MOSL[[row]][[j]][[k]])<=N.mosl-set.size(PLOS))) {
+					if ((Term[[row]][[j]][[k]] == 0) & (!is.na(MOSL[[row]][[j]][[k]][[1]])) & (length(MOSL[[row]][[j]][[k]])<=N.mosl-set.size(PLOS)) & !is.element(TRUE,is.element(unique(MOSL[[row]][[j]][[k]]),looplist.elements(PLOS)))) {
 						return(list(MOSL[[row]][[j]][[k]],k))
 						}
 					}
@@ -439,9 +488,9 @@ enumerate.SOSL <- function(MOSL,N) {
 			if (length(loop) >1 ) {
 				for (edge in 1:(length(loop))) {
 					if (edge < length(loop)) {
-						lprod <- lprod * C[loop[edge],loop[edge+1]]
+						lprod <- lprod * C[loop[edge+1],loop[edge]]
 						} else {
-							lprod <- lprod * C[loop[edge],loop[1]]
+							lprod <- lprod * C[loop[1],loop[edge]]
 						 	}
 					 }
 				}
@@ -499,7 +548,7 @@ enumerate.SOSL <- function(MOSL,N) {
 # CM: a community matrix
 # path: a path vactor through the community matrix
 		path.prod <- function(CM,path) {
-			pprod <- CM[path[1],path[2]]
+			pprod <- CM[path[2],path[1]]
 			if (length(path) == 2) {
 				if (path[1] == path[2]) {
 					return(1)
@@ -507,7 +556,7 @@ enumerate.SOSL <- function(MOSL,N) {
 				return(pprod)
 				}
 			for (edge in 2:(length(path)-1) ) {
-				pprod <- pprod * CM[path[edge],path[edge+1]]
+				pprod <- pprod * CM[path[edge+1],path[edge]]
 				}	
 
 			return(pprod)
@@ -575,9 +624,9 @@ enumerate.SOSL <- function(MOSL,N) {
 		}
 	
 	if (out) {
-		out.cm(t(CEM))
+		out.cm(CEM)
 		}
-	return(t(CEM))
+	return(CEM)
 	
 	# end make.cem()
 	}
